<?php
/*
 * $Id: avreloaded.php.in 711 2008-01-22 02:04:16Z Fritz Elfert $
 *
 * AllVideos Reloaded for Joomla! 1.5
 *
 * Author: Fritz Elfert
 * Copyright 2007 by Fritz Elfert
 *
 * Inspired by and partially based on:
 *
 *   The "AllVideos" Plugin for Joomla 1.0.x - Version 2.4
 *   Authors: Fotis Evangelou - George Chouliaras
 *   Copyright (c) 2006 JoomlaWorks.gr - http://www.joomlaworks.gr
 */

defined('_JEXEC') or die("Direct Access Is Not Allowed");
jimport('joomla.plugin.plugin');

class plgContentAvreloaded extends JPlugin {

    var $_version = '1.0beta2';
    var $_rev = '$Revision: 711 $';
    // Our standard header
    var $_beg = "<!-- AllVideos Reloaded Plugin (%s) starts here\n--><%s id=\"@DIVID@\"%s class=\"%s\">\n";
    // Our standard trailer
    var $_end = "</%s><!--\nAllVideos Reloaded Plugin (%s) ends here -->";
    // The height of the builtin player's controls
    var $_ctrlheight = 20;
    // "Web" color names as specified by HTML 4.01
    var $_w3colors = array(
        'aqua'    => 0x00ffff,
        'black'   => 0,
        'blue'    => 0xff,
        'fuchsia' => 0xff00ff,
        'green'   => 0x8000,
        'grey'    => 0x808080,
        'lime'    => 0xff00,
        'maroon'  => 0x800000,
        'navy'    => 0x80,
        'olive'   => 0x808000,
        'purple'  => 0x800080,
        'red'     => 0xff0000,
        'silver'  => 0xc0c0c0,
        'teal'    => 0x8080,
        'white'   => 0xffffff,
        'yellow'  => 0xffff00,
    );
    // Our include directory
    var $_rdir = null;
    // Our resource location URI
    var $_rloc = null;
    // Our local media location
    var $_mloc = null;
    // Our version tag
    var $_vtag = null;
    // DB-Support? (com_avreloaded available and enabled) available
    var $_dbok = 0;

    ///// Content plugin API interface starts here

    /**
     * Constructor
     *
     * For php4 compatability we must not use the __constructor as a constructor for plugins
     * because func_get_args ( void ) returns a copy of all passed arguments NOT references.
     * This causes problems with cross-referencing necessary for the observer design pattern.
     *
     * @param object $subject The object to observe
     * @param object $params  The object that holds the plugin parameters
     * @param int    $special Used internally
     * @since 1.5
     */
    function plgContentAvreloaded(& $subject, $params, $special = 0) {
        parent::__construct($subject, $params);
        $this->_init();
    }

    /**
     * Main prepare content method
     * Method is called by the view
     *
     * @param       object          The article object.  Note $article->text
     *                              is also available
     * @param       object          The article params
     * @param       int             The 'page' number
     */
    function onPrepareContent(&$article, &$params, $limitstart) {
        $article->text = $this->_doSubstitution($article->text);
    }

    ///// Content plugin API interface ends here

    ///// Alternative API: Call by value, returning the result.
    ///// For use in custom modules.
    function onAvReloadedGetVideo($string) {
        return $this->_doSubstitution($string);
    }
    ///// Alternative API ends here

    /**
     * Local initialization
     */
    function _init() {
        $mparams =& JComponentHelper::getParams('com_media');
        $this->_mloc = JURI::root(true).'/'.$mparams->get('image_path', 'images/stories').'/';
        $this->_rdir = JPATH_PLUGINS.DS.'content'.DS.'avreloaded'.DS;
        $this->_rloc = JURI::root(true).'/plugins/content/avreloaded/';
        $this->_vtag = 'v'.$this->_version.'.'.preg_replace('#\D#', '', $this->_rev);

        $tags = null;
        // Check for our corresponding component which owns the db tables.
        if (JComponentHelper::isEnabled('com_avreloaded', true)) {
            $db = &JFactory::getDBO();
            $query = 'SELECT * FROM #__avreloaded_tags';
            @$db->setQuery($query);
            @$tags = $db->loadAssoc();
            $this->_dbok = is_array($tags);
        }
        if (!$this->_dbok) {
            include($this->_rdir.'tags.php');
        }
        if (!is_array($tags)) {
            JError::raiseError(500, JText::_('ERR_TAGS'));
        }
        $this->tags = $tags;
    }

    /**
     * Helper function for adjusting local URLs.
     *
     * @param loc The location to adjust.
     * @return The adjusted location
     * @access private
     */
    function _adjustLoc($loc) {
        if (empty($loc))
            return '';
        return ((JString::strpos($loc, '/') === 0) ? $loc : ($this->_mloc.$loc));
    }

    /**
     * Helper function for emulating htmlspecialchars_decode
     * on PHP4.
     *
     * @param string The string to decode.
     * @return The decoded string.
     * @access private
     */
    function _htsdecode($string) {
        if (function_exists('htmlspecialchars_decode')) {
            return htmlspecialchars_decode($string);
        } else {
            return strtr($string, array_flip(get_html_translation_table(HTML_SPECIALCHARS)));
        }
    }

    /**
     * The actual work horse of the plugin. Here the text gets scanned for known
     * tags and then these tags get replaced by the necessary code to embed the
     * video player.
     *
     * @param text The article text to be scanned/replaced
     * @return The text, with all known tags replaced.
     * @access private
     */
    function _doSubstitution($text) {
        // Joomla's plugin-installer does not handle separate language files for
        // backend and frontend if installing a plugin (kind of silly, as it works
        // with components). Instead, it installs the language files always into the
        // JPATH_ADMINISTRATOR/languages. Therefore we have to specify this path
        // explicitely.
        JPlugin::loadLanguage('plg_content_avreloaded', JPATH_ADMINISTRATOR);
        // Get the plugin parameters
        $plugin = &JPluginHelper::getPlugin('content', 'avreloaded');
        $params = new JParameter($plugin->params);
        $cfg = array();

        // general
        $cfg['rloc']            = $this->_rloc;
        $cfg['avcss']           = $params->get('avcss', 'allvideos');
        $cfg['tmargin']         = $params->get('tmargin', 8);
        $cfg['bmargin']         = $params->get('bmargin', 8);
        $cfg['valign']          = $params->get('valign', 'center');
        $cfg['width']           = $params->get('width', 400);
        $cfg['height']          = $params->get('height', 320);
        $cfg['vloc']            = $this->_adjustLoc($params->get('vdir', 'videos').'/');
        $cfg['wmode']           = $params->get('wmode', 'window');
        $cfg['bgcolor']         = $this->_fmtColor($params->get('bgcolor', '#FFFFFF'), '#');
        $cfg['legacy']          = $params->get('legacy', '0');
        $cfg['autostart']       = $this->_fmtBool($params->get('autostart', '0'));
        $cfg['usefullscreen']   = $this->_fmtBool($params->get('usefullscreen', '0'));
        $cfg['showdigits']      = $this->_fmtBool($params->get('showdigits', '1'));
        $cfg['showicons']       = $this->_fmtBool($params->get('showicons', '1'));
        $cfg['showstop']        = $this->_fmtBool($params->get('showstop', '0'));
        $cfg['showdownload']    = $this->_fmtBool($params->get('showdownload', '0'));
        $cfg['showvolume']      = $this->_fmtBool($params->get('showvolume', '1'));
        $cfg['enablejs']        = $this->_fmtBool($params->get('enablejs', '0'));
        $cfg['pbgcolor']        = $this->_fmtColor($params->get('pbgcolor', '#FFFFFF'), '0x');
        $cfg['pfgcolor']        = $this->_fmtColor($params->get('pfgcolor', '#000000'), '0x');
        $cfg['phicolor']        = $this->_fmtColor($params->get('phicolor', '#000000'), '0x');
        $cfg['psccolor']        = $this->_fmtColor($params->get('psccolor', '#000000'), '0x');
        $cfg['logo']            = $params->get('logo', '-1');
        $cfg['logo']            = $this->_adjustLoc(($cfg['logo'] == '-1') ? '' : $cfg['logo']);
        if ($cfg['logo'] == '') unset($cfg['logo']);
        $cfg['screenmode']      = $params->get('screenmode', 'default');
        $cfg['plsize']          = $params->get('plsize', 30);
        $cfg['stretch']         = $params->get('stretch', 0);
        switch ($cfg['stretch']) {
        case 0:
            $cfg['stretch'] = 'false';
            break;
        case 1:
            $cfg['stretch'] = 'true';
            break;
        case 2:
            $cfg['stretch'] = 'fit';
            break;
        case 3:
            $cfg['stretch'] = 'none';
            break;
        }
        // iFilm - Metacafe - GameTrailers only
        $cfg['iwidth']          = $params->get('iwidth', 400);
        $cfg['iheight']         = $params->get('iheight', 320);
        $cfg['mwidth']          = $params->get('mwidth', 400);
        $cfg['mheight']         = $params->get('mheight', 320); 
        $cfg['cwidth']          = $params->get('width', 400);
        $cfg['cheight']         = $params->get('height', 320);  
        // audio only
        $cfg['awidth']          = $params->get('awidth', 300);
        $cfg['aheight']         = $params->get('aheight', 20);
        $cfg['aloc']            = $this->_adjustLoc($params->get('adir', 'audio').'/');
        $cfg['site']            = JURI::root(false);
        $cfg['linkfromdisplay'] = 'false';
        $cfg['linktarget']      = '_blank';
        // prepend and append code
        $vstyle = '';
        $container = 'span';
        if ($cfg['legacy']) {
            $container = 'div';
            $vstyle = ' style="clear:both; text-align: '.$cfg['valign'].'; margin-top: '.
                $cfg['tmargin'].'px; margin-bottom: '.$cfg['bmargin'].'px;"';
        }
        $start = sprintf($this->_beg, $this->_vtag, $container, $vstyle, $cfg['avcss']);
        $end = sprintf($this->_end, $container, $this->_vtag);
        $found = 0;
        $needswfo = 0;
        $needavrjs = 0;

        static $divid;
        if (!isset($divid)) {
            $divid = 0;
        }

        foreach ($this->tags as $key => $fmt) {
            // Speedup non-matching case
            if (strpos($text, $key) > 0) {
                // The actual regex stuff
                $re = '#{'.$key.'(\s+[a-z]+\s*=\s*(?:"|&quot;)[^}]*(?:"|&quot;))*}([^{]+){/'.$key.'}#m';
                if (preg_match_all($re, $text, $matches, PREG_PATTERN_ORDER) > 0) {
                    // load the matching format
                    $fmtdef = null;
                    if ($this->_dbok) {
                        $db = &JFactory::getDBO();
                        $query = 'SELECT * FROM #__avreloaded_formats where id = ' . $key;
                        $db->setQuery($query);
                        $fmtdef = $db->loadAssoc();
                    } else {
                        $fn = $this->_rdir.'fmt-'.$fmt.'.php';
                        include($fn);
                        // Validate loaded file
                        if ((!is_array($formats)) || (!is_array($formats[$key]))) {
                            JError::raiseError(500, JText::_('ERR_FORMAT'), $fn);
                        }
                        $fmtdef = $formats[$key];
                    }
                    if (!isset($fmtdef['player'])) {
                        JError::raiseError(500, JText::_('ERR_FORMAT'), $fn);
                    }
                    $player = $fmtdef['player'];
                    $avp = null;
                    if (isset($fmtdef['avp'])) {
                        $avp = $fmtdef['avp'];
                        if (!is_array($avp)) {
                            JError::raiseError(500, JText::_('ERR_FORMAT'), $fn);
                        }
                        foreach ($avp as $old => $new) {
                            // If old matches exactly one @...@ and
                            // new does not contain any @...@, we delay replacement
                            // and set a cfg var instead.
                            $omatch = null;
                            if (preg_match('#^@([A-Z]+)@$#', $old, $omatch) &&
                                (preg_match('#@[A-Z]+@#', $new) == 0)) {
                                    $ckey = strtolower($omatch[1]);
                                    $cfg[$ckey] = $new;
                            } else {
                                $player = str_replace($old, $new, $player);
                            }
                        }
                    }

                    // The actual replacement of tags
                    $i = 0;
                    foreach ($matches[0] as $match) {
                        $parms = JString::trim($this->_htsdecode($matches[1][$i]));
                        // tplr is a reference first
                        $tplr = &$player;
                        $tcfg = $cfg;
                        $tstart = $start;
                        $tend = $end;
                        $tcfg['divid'] = 'avreloaded'.$divid++;
                        if (JString::strlen($parms)) {
                            // If individual parameters are specified,
                            // make tplr a deep copy ..
                            $tplr = $player;
                            $this->_parseParams($parms, $tcfg, $tstart, $tend);
                        }

                        $vcode = $matches[2][$i];
                        // If a ripper is defined, execute it
                        if (isset($fmtdef['ripper'])) {
                            $rcfg = $fmtdef['ripper'];
                            $tmp = $tcfg;
                            $rcfg['url'] = $this->_buildCode($vcode, $tmp, $rcfg['url']);
                            // save old code, in case it's needed as well
                            $tcfg['ocode'] = $vcode;
                            $vcode = $this->_ripper($rcfg);
                        }
                        $tcfg['idseq'] = $divid;
                        if (((0 + $tcfg['width']) >= 214) && ((0 + $tcfg['height']) >= 137)) {
                            // The Adobe express install needs at least a flash
                            // window of 214x137 pixels.
                            $tcfg['xpinst'] = "'".$this->_rloc."expressinstall.swf'";
                        } else {
                            $tcfg['xpinst'] = 'false';
                        }
                        $code = $tstart.$tplr.$tend;
                        $code = $this->_buildCode($vcode, $tcfg, $code);
                        if (isset($tcfg['enablejs']) && ($tcfg['enablejs'] == 'true')) {
                            $needavrjs = 1;
                        }
                        if (is_int(strpos($code, 'swfobject.'))) {
                            $needswfo = 1;
                        }
                        $text = str_replace($match, $code, $text );
                        $found = 1;
                        $i++;
                    }
                }
            }
        }
        if ($needswfo) {
            $d = &JFactory::getDocument();
            $d->addScript($this->_rloc.'swfobject.js');
            if ($needavrjs) {
                $d->addScript($this->_rloc.'avreloaded.js');
            }
        }
        return $text;
    }

    /**
     * Parse a string, containing multiple parameters, each in the form
     * key="value" and assign them to an assoziative array.
     *
     * @param parms
     * @param cfg
     * @param start
     * @param end
     *
     * @access private
     */
    function _parseParams($parms, &$cfg, &$start, &$end) {
        $legacy_override = 0;
        $style_override = '';
        if (preg_match_all('#\s*([a-z]+)\s*=\s*"([^}"]*)"\s*#', $parms, $matches)) {
            $i = 0;
            foreach ($matches[1] as $key) {
                switch ($key) {
                case 'bgcolor':
                    $cfg[$key] = $this->_fmtColor($matches[2][$i], '#');
                    break;
                case 'pbgcolor':
                case 'pfgcolor':
                case 'phicolor':
                case 'psccolor':
                    $cfg[$key] = $this->_fmtColor($matches[2][$i], '0x');
                    break;
                case 'autostart':
                case 'usefullscreen':
                case 'showdigits':
                case 'showicons':
                case 'showstop':
                case 'showdownload':
                case 'showvolume':
                    $cfg[$key] = $this->_fmtBool($matches[2][$i]);
                    break;
                case 'style':
                    $style_override = $matches[2][$i];
                    break;
                case 'legacy':
                    $legacy_override = 1;
                    $cfg[$key] = $this->_fmtBool($matches[2][$i]);
                    break;
                default:
                    $cfg[$key] = $matches[2][$i];
                    break;
                }
                $i++;
            }
            if ($legacy_override) {
                $style = '';
                $container = 'span';
                if ($cfg['legacy'] == 'true') {
                    $style = ' style="clear:both; text-align: '.$cfg['valign'].'; margin-top: '.
                        $cfg['tmargin'].'px; margin-bottom: '.$cfg['bmargin'].'px;"';
                    $container = 'div';
                }
                $start = sprintf($this->_beg, $this->_vtag, $style, $cfg['avcss']);
                $end = sprintf($this->_end, $container, $this->_vtag);
            }
            if ($style_override) {
                $start = sprintf($this->_beg, $this->_vtag, ' style="'.$style_override.'"', $cfg['avcss']);
            }
        }
    }

    /**
     * Replace variables in a player template
     * For every @NAME@ tag, lookup the lowercaes NAME in 
     * the supplied config array and - if found - replace it by the stored
     * value. Furthermode, If a sequence @IF(NAME)@..content..@/IF@ is
     * found, replace that sequence with an empty string, if the variable
     * specified by lowercase NAME is unset or not 'true'. Similar, if
     * @IFS(NAME)@..content..@/IFS@ is found, replace that sequence with
     * an empty string, if the variable specified by lowercase NAME is unset.
     *
     * @param code The content of the element's tag.
     * @param cfg  The current config to be used.
     * @param pltmpl The player template to be used.
     *
     * @return string The player template with all matching patterns replaced.
     *
     * @access private
     */
    function _buildCode($code, $cfg, $pltmpl) {
        $matches = null;
        //$dwvarname = '_dummy';
        //$dhvarname = '_dummy';
        if ((!isset($cfg['displayheight'])) && (!isset($cfg['displaywidth']))) {
            // If the user has overridden displayheight or displaywidth already
            // from within the tag, don't touch it!
            switch ($cfg['screenmode']) {
            case 'coverlay':
                $cfg['displayheight'] = $cfg['height'];
                break;
            case 'plbottom':
                $cfg['displayheight'] = $cfg['height'] - ($this->_ctrlheight + $cfg['plsize']);
                break;
            case 'plright':
                $cfg['displaywidth'] = $cfg['width'] - $cfg['plsize'];
                break;
            case 'floatcplright':
                $cfg['displayheight'] = $cfg['height'];
                $cfg['displaywidth'] = $cfg['width'] - $cfg['plsize'];
                break;
            }
        }
        /*
        if (isset($cfg['displayheight'])) {
            $dhvarname = 'displayheight';
        } else {
            $cfg['displayheight'] = '0';
        }
        if (isset($cfg['displaywidth'])) {
            $dwvarname = 'displaywidth';
        } else {
            $cfg['displaywidth'] = '0';
        }
        */
        // First handle conditionals ...
        if (preg_match_all('#@IF\(([A-Z]+)\)@(.+)@/IF@#sU', $pltmpl, $matches, PREG_PATTERN_ORDER)) {
            $i = 0;
            foreach ($matches[0] as $match) {
                $key = strtolower($matches[1][$i]);
                $inner = '';
                if ((isset($cfg[$key])) && ($this->_fmtBool($cfg[$key]) == 'true')) {
                    $inner = $matches[2][$i];
                }
                $pltmpl = str_replace($match, $inner, $pltmpl);
                $i++;
            }
        }
        if (preg_match_all('#@IFS\(([A-Z]+)\)@(.+)@/IFS@#sU', $pltmpl, $matches, PREG_PATTERN_ORDER)) {
            $i = 0;
            foreach ($matches[0] as $match) {
                $key = strtolower($matches[1][$i]);
                $inner = '';
                if (isset($cfg[$key])) {
                    $inner = $matches[2][$i];
                }
                $pltmpl = str_replace($match, $inner, $pltmpl);
                $i++;
            }
        }
        // ... then handle regular replacements
        if (preg_match_all('#@([A-Z]+(?:![dy])?)@#', $pltmpl, $matches, PREG_PATTERN_ORDER)) {
            $i = 0;
            foreach ($matches[0] as $match) {
                $key = strtolower($matches[1][$i]);
                $boolfmtoverride = null;
                if (strpos($key, '!d')) {
                    $boolfmtoverride = 'ds';
                    $key = str_replace('!d', '', $key);
                }
                if (strpos($key, '!y')) {
                    $boolfmtoverride = 'yn';
                    $key = str_replace('!y', '', $key);
                }
                if (isset($cfg[$key])) {
                    $val = $cfg[$key];
                    if ($boolfmtoverride) {
                        $val = $this->_fmtBool($val, $boolfmtoverride);
                    }
                    $pltmpl = str_replace($match, $val, $pltmpl);
                }
                $i++;
            }
        }
        //$pltmpl = str_replace('@DWIDTHVARNAME@', $dwvarname, $pltmpl);
        //$pltmpl = str_replace('@DHEIGHTVARNAME@', $dhvarname, $pltmpl);
        return str_replace('@CODE@', $code, $pltmpl);
    }

    /**
     * Format a boolean value
     *
     * @param value The input to be formatted.
     *                Possible input formats:
     *                  - A numeric (int or string) (0 == false, !0 == true)
     *                  - A string in the form 'true' or 'false'
     * @param fmt   The desired output format:
     *                'bs'  return a string 'true' or 'false'
     *                'ds'  return a string '1' or '0'
     *                'yn'  return a string 'yes' or 'no'
     *
     * @return string
     * @access private
     */
    function _fmtBool($value, $fmt = 'bs') {
        if (is_numeric($value)) {
            $value = ((1 + $value) != 1);
        }
        if (is_string($value)) {
            $value = (strtolower($value) == 'true');
        }
        switch ($fmt) {
        case 'bs':
            // return boolean string
            return ($value) ? 'true' : 'false';
            break;
        case 'ds':
            // return decimal string
            return ($value) ? '1' : '0';
            break;
        case 'yn':
            // return yes/no string
            return ($value) ? 'yes' : 'no';
            break;
        }
    }

    /**
     * Retrieves the real url of a media resource.
     * First, the content of a given url is fetched, then
     * this content is matched against a given regex in order
     * to retrieve a fragment which contains the actual media
     * url.
     *
     * @param ripcfg An associative array of parameters.
     *
     * @return The extracted media URL.
     * @access private
     */
    function _ripper($ripcfg) {
        $url = str_replace(' ', '%20', $ripcfg['url']);
        $resp = '';
        if (ini_get('allow_url_fopen') == '1') {
            $resp = @file_get_contents($url);
        } else if (function_exists('curl_init')) {
            // try fallback to cURL
            $ch = curl_init();
            curl_setopt($ch, CURLOPT_URL, $url);
            curl_setopt($ch, CURLOPT_HEADER, 0);
            curl_setopt($ch, CURLOPT_BINARYTRANSFER, 1);
            curl_setopt($ch, CURLOPT_FOLLOWLOCATION, 1);
            curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
            curl_setopt($ch, CURLOPT_MAXREDIRS, 10);
            curl_setopt($ch, CURLOPT_TIMEOUT, 30);
            if (!($resp = curl_exec($ch)))
                $resp = '';
            curl_close($ch);
        }
        if (strlen($resp)) {
            if ($ripcfg['urldecode']) {
                $resp = urldecode($resp);
            }
            $resp = $this->_htsdecode($resp);
            if (preg_match($ripcfg['regex'], $resp, $matches)) {
                return $matches[1];
            }
            $this->_err(JText::sprintf('ERR_MATCH_URL',
                htmlspecialchars($url),
                htmlspecialchars($resp),
                htmlspecialchars($ripcfg['regex'])));
        } else
            $this->_err(JText::sprintf('ERR_FETCH_URL',
                htmlspecialchars($url)));
        return '';
    }

    /**
     * Displays an error message inline
     */
    function _err($msg) {
        global $mainframe;
        $mainframe->enqueueMessage($msg, 'error');
    }

    /**
     * Format a color value.
     *
     * @param  clr The color value to be formatted.
     *               Possible input formats:
     *                 - an int color value
     *                 - a string in '0xxxx' notation (like in javascript)
     *                 - a string in #RRGGBB notation (HTML, CSS)
     *                 - a string in #RGB notation (CSS)
     *                 - a string in rgb(r,g,b) notation (CSS)
     *                 - a string with a color name
     * @param  fmt The output color format:
     *               '0x'  return a string in '0xxx' notation
     *               '#'   return a string in #RRGGBB notation
     *               'rgb' return a string in rgb(r,g,b) notation
     *
     * @return string
     * @access private
     */
    function _fmtColor($clr, $fmt = '0x') {
        $iclr = -1;
        if (is_int($clr)) {
            if ($clr <= 0xFFFFFF)
                $iclr = $clr;
        }
        if (is_string($clr)) {
            $match = null;
            $clr = trim($clr);
            // #xxx css notation and #xxxxxx html/css notation
            if (preg_match('/^#([\da-f]{3,6})$/i', $clr, $match)) {
                switch (strlen($match[1])) {
                case 3:
                    $iclr = intval(sprintf('%s%s%s%s%s%s',
                        $match[1][0], $match[1][0],
                        $match[1][1], $match[1][1],
                        $match[1][2], $match[1][2]), 16);
                    break;
                case 6:
                    $iclr = intval($match[1], 16);
                    break;
                }
            }
            // 0x... javascript notation
            if (($iclr == -1) && preg_match('/^0x([0-9a-f]{1,6})$/i', $clr, $match)) {
                $iclr = intval($match[1], 16);
            }
            // rgb(d,d,d) css notation
            if (($iclr == -1) && preg_match('/^rgb\s*\(\s*([-+]?\d+)\s*,\s*([-+]?\d+)\s*,\s*([-+]?\d+)\s*\)/', $clr, $match)) {
                $r = intval($match[1]); if ($r < 0) $r = 0; if ($r > 255) $r = 255;
                $g = intval($match[2]); if ($g < 0) $g = 0; if ($g > 255) $g = 255;
                $b = intval($match[3]); if ($b < 0) $b = 0; if ($b > 255) $b = 255;
                $iclr = ($r << 16) | ($g << 8) | $b;
            }
            // html 4.01 color names
            if ($iclr == -1) {
                $cn = strtolower($clr);
                if (isset($this->_w3colors[$cn])) {
                    $iclr = $this->_w3colors[$cn];
                }
            }
            if ($iclr == -1) {
                $this->_err(JText::sprintf('ERR_COLORVAL', htmlspecialchars($clr)));
                return '';
            }
            switch ($fmt) {
            case '0x':
                return sprintf("0x%06X", $iclr);
            case '#':
                return sprintf("#%06X", $iclr);
            case 'rgb':
                return sprintf("rgb(%d,%d,%d)", $iclr >> 16, ($iclr >> 8) & 0xff, $iclr & 0xff);
            default:
                JError::raiseError(500, JText::_('ERR_COLORFMT'), $fmt);
            }
        }
    }

}

